library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity ER_Project is port (
  clk : in std_logic;
  pwm : out std_logic_vector(1 downto 0);
  sonar_trig : out STD_LOGIC;
  sonar_echo : in  STD_LOGIC;
  Diz7segs   : out std_logic_vector(6 downto 0);
  Unit7segs  : out std_logic_vector(6 downto 0)
    );
end entity;

architecture Movemotor of ER_Project is

component diviseur is port (
	clk : in std_logic;
	eno : out std_logic
--	q : out std_logic_vector(23 downto 0)
	);
end component;

component cmpt_hzt is port (
	clk, en : in std_logic;
	eno : out std_logic;
	s_hzt : out std_logic_vector(7 downto 0)
	);
end component;

component cmpt_vtc is port (
	clk, en : in std_logic;
	s_vtc : out std_logic_vector(7 downto 0)
	);
end component;

-- component of sonar
component affichage2digits is
    port (  
            I_in8    : in  std_logic_vector(7 downto 0);
				-- Ordre : gfedcba
            Diz7segs,Unit7segs  : out std_logic_vector(6 downto 0)
    );
  end component affichage2digits;

 signal timer : natural range 0 to 3900 := 0;
 signal eno : std_logic;
 signal s_cmpt8 : std_logic_vector(7 downto 0);
 signal s_eno, s_eno1 : std_logic;
 signal ref1 : std_logic_vector(7 downto 0);
 signal ref2 : std_logic_vector(7 downto 0);
 
 -- signal of sonar
    signal count            : std_logic_vector(16 downto 0) := (others => '0');
    signal centimeters      : std_logic_vector(15 downto 0) := (others => '0');
    signal centimeters_ones : std_logic_vector(3 downto 0)  := (others => '0');
    signal centimeters_tens : std_logic_vector(3 downto 0)  := (others => '0');
    signal output_ones      : std_logic_vector(3 downto 0)  := (others => '0');
    signal output_tens      : std_logic_vector(3 downto 0)  := (others => '0');
    signal digit            : std_logic_vector(3 downto 0)  := (others => '0');
    signal echo_last        : std_logic := '0';
    signal echo_synced      : std_logic := '0';
    signal echo_unsynced    : std_logic := '0';
    signal waiting          : std_logic := '0'; 
    signal seven_seg_count  : std_logic_vector(15 downto 0) := (others => '0');
	 
begin 

div : diviseur port map (
	clk => clk, eno => s_eno
	);
cmpt1 : cmpt_hzt port map (
	clk => clk, en => s_eno, eno => s_eno1, s_hzt => ref1
	);
cmpt2 : cmpt_vtc port map (
	clk => clk, en => s_eno1, s_vtc => ref2
	);

-- sonar
  affichage : affichage2digits port map(
       I_in8(3 downto 0) =>  output_ones,
		 I_in8(7 downto 4) =>  output_tens,
       Diz7segs => Diz7segs,
		 Unit7segs => Unit7segs 
  );

 -- realisation 78 us
 -- process of Servo
 process(clk)
  begin
    if rising_edge(clk) then
      if timer = 0 then
        timer <= 3900;
        eno <= '1';
      else
        timer <= timer - 1;
		  eno <='0';
      end if;
    end if;
  end process;
 
  cmpt8:process(clk) begin
     if rising_edge(clk) then
	    if eno = '1' then
		   s_cmpt8 <= s_cmpt8 + 1;
		 end if;
	  end if;
	end process; 
	
-- process of sonar
process(clk)
    begin
        if rising_edge(clk) then
            if waiting = '0' then
            --    if count = 1000 then -- Assumes 100MHz
				   if count = 500 then -- Assumes 50MHz
                   -- After 10us then go into waiting mode
                   sonar_trig <= '0';
                   waiting    <= '1';
                   count       <= (others => '0');
                else
                   sonar_trig <= '1';
                   count <= count+1;
                end if;
            elsif echo_last = '0' and echo_synced = '1' then
                -- Seen rising edge - start count
                count       <= (others => '0');
                centimeters <= (others => '0');
                centimeters_ones <= (others => '0');
                centimeters_tens <= (others => '0');
            elsif echo_last = '1' and echo_synced = '0' then
                -- Seen falling edge, so capture count
                output_ones <= centimeters_ones; 
                output_tens <= centimeters_tens; 
 --           elsif count = 2900*2 -1 then
             elsif count = 1450*2 -1 then
                -- advance the counter
                if centimeters_ones = 9 then
                    centimeters_ones <= (others => '0');
                    centimeters_tens <= centimeters_tens + 1;
                else
                    centimeters_ones <= centimeters_ones + 1;
                end if;
                centimeters <= centimeters + 1;
                count <= (others => '0');
                if centimeters = 3448 then
                    -- time out - send another pulse
                    waiting <= '0';
                end if;
            else
                count <= count + 1;                
            end if;

            echo_last     <= echo_synced;
            echo_synced   <= echo_unsynced;
            echo_unsynced <= sonar_echo;
        end if;
        
    end process;

--comparaison svm hrz
 process(s_cmpt8)
  begin	
	if (s_cmpt8 < ref1) then
		pwm(0) <= '1';
	else
		pwm(0) <= '0';
	end if;
  end process;
  
  
  --comparaison svm vtc
 process(s_cmpt8)
  begin	
	if (s_cmpt8 < ref2) then
		pwm(1) <= '1';
	else
		pwm(1) <= '0';
	end if;
  end process;
  
end Movemotor;











-- library for servo
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
entity diviseur is port (
	clk : in std_logic;
	eno : out std_logic
--	q : out std_logic_vector(23 downto 0)
	);
end entity diviseur;

architecture arch of diviseur is
signal cmpt : std_logic_vector(23 downto 0);
begin
  process (clk) begin
    if rising_edge(clk) then
	   cmpt <= cmpt+1;
	 end if;
  end process;
  eno <= '1' when cmpt = x"FFFFFF" else
         '0';
end arch;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
entity cmpt_hzt is port (
	clk, en : in std_logic;
	eno : out std_logic;
	s_hzt : out std_logic_vector(7 downto 0)
	);
end entity cmpt_hzt;

architecture arch of cmpt_hzt is
  signal cmpt : std_logic_vector(7 downto 0) := x"0A";
begin
  process (clk) begin
    if rising_edge(clk) then
	   if en='1' then
	     if cmpt < x"1F" then
		    cmpt <= cmpt+1;
		  else
		    cmpt <= x"08";
		  end if;
		end if;
	 end if;
  end process;
  eno <= '1' when cmpt = x"1F" else
         '0';
  s_hzt <= cmpt;
end arch;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
entity cmpt_vtc is port (
	clk, en : in std_logic;
	s_vtc : out std_logic_vector(7 downto 0)
	);
end entity;

architecture arch of cmpt_vtc is
  signal cmpt : std_logic_vector(7 downto 0) := x"0A";
begin
  process (clk) begin
    if rising_edge(clk) then
	   if en='1' then
	     if cmpt < x"14" then
		    cmpt <= cmpt+1;
		  else
		    cmpt <= x"0A";
		  end if;
		end if;
	 end if;
  end process;
  s_vtc <= cmpt;
end arch;












-- library for sonar
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
-- zero pour allumer les LEDs
entity transcod7segs is
    port (  
            I_in4    : in  std_logic_vector(3 downto 0);
				-- Ordre : gfedcba
            Q_7segs  : out std_logic_vector(6 downto 0)
    );
end transcod7segs;

architecture arch of transcod7segs is begin
  with I_in4 select
    Q_7SEGS <= "1000000" when "0000",
               "1111001" when "0001",
               "0100100" when "0010",
               "0110000" when "0011",
               "0011001" when X"4",
               "0010010" when X"5",
               "0000010" when X"6",
               "1111000" when X"7",
               "0000000" when X"8",
               "0010000" when X"9",
               "0001000" when X"A",
               "0000011" when X"B",
               "1000110" when X"C",
               "0100001" when X"D",
               "0000110" when X"E",
               "0001110" when others; --X"F"
end arch;

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
-- zero pour allumer les segments
entity affichage2digits is
    port (  
            I_in8    : in  std_logic_vector(7 downto 0);
				-- Ordre : gfedcba
            Diz7segs,Unit7segs  : out std_logic_vector(6 downto 0)
    );
end affichage2digits;

architecture arch_affichage2digits of affichage2digits is
component transcod7segs is
    port (  
            I_in4    : in  std_logic_vector(3 downto 0);
				-- Ordre : gfedcba
            Q_7segs  : out std_logic_vector(6 downto 0)
    );
end component transcod7segs;
 
begin
  unite : transcod7segs port map (
     I_in4 => I_in8(3 downto 0),
	  Q_7segs => Unit7segs
	  );
  dizaine : transcod7segs port map (
     I_in4 => I_in8(7 downto 4),
	  Q_7segs => Diz7segs
	  );
end arch_affichage2digits;